"use strict";
const __rslib_import_meta_url__ = /*#__PURE__*/ (function () {
  return typeof document === 'undefined'
    ? new (require('url'.replace('', '')).URL)('file:' + __filename).href
    : (document.currentScript && document.currentScript.src) ||
      new URL('main.js', document.baseURI).href;
})();
;
// The require scope
var __webpack_require__ = {};

/************************************************************************/
// webpack/runtime/define_property_getters
(() => {
__webpack_require__.d = function(exports, definition) {
	for(var key in definition) {
        if(__webpack_require__.o(definition, key) && !__webpack_require__.o(exports, key)) {
            Object.defineProperty(exports, key, { enumerable: true, get: definition[key] });
        }
    }
};
})();
// webpack/runtime/has_own_property
(() => {
__webpack_require__.o = function (obj, prop) {
	return Object.prototype.hasOwnProperty.call(obj, prop);
};

})();
// webpack/runtime/make_namespace_object
(() => {
// define __esModule on exports
__webpack_require__.r = function(exports) {
	if(typeof Symbol !== 'undefined' && Symbol.toStringTag) {
		Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });
	}
	Object.defineProperty(exports, '__esModule', { value: true });
};

})();
/************************************************************************/
var __webpack_exports__ = {};
// ESM COMPAT FLAG
__webpack_require__.r(__webpack_exports__);

// EXPORTS
__webpack_require__.d(__webpack_exports__, {
  createApp: () => (/* reexport */ createApp),
  createCommandsPlugin: () => (/* reexport */ createCommandsPlugin),
  SUPPORTED_ATTACHMENT_TYPES: () => (/* reexport */ SUPPORTED_ATTACHMENT_TYPES),
  VercelChatModelAdapter: () => (/* reexport */ VercelChatModelAdapter),
  UnsupportedAttachmentsException: () => (/* reexport */ UnsupportedAttachmentsException),
  createMockChatModel: () => (/* reexport */ createMockChatModel),
  PerformanceMarks: () => (/* reexport */ PerformanceMarks),
  getReferenceStorage: () => (/* reexport */ getReferenceStorage),
  loggingPlugin: () => (/* reexport */ loggingPlugin),
  PerformanceTimeline: () => (/* reexport */ PerformanceTimeline),
  isAttachmentTypeSupported: () => (/* reexport */ isAttachmentTypeSupported)
});

;// CONCATENATED MODULE: external "@callstack/byorg-utils"
const byorg_utils_namespaceObject = require("@callstack/byorg-utils");
;// CONCATENATED MODULE: ./src/constants.ts
const PerformanceMarks = {
    processMessages: 'processMessages',
    middlewareBeforeHandler: 'middleware:beforeHandler',
    middlewareAfterHandler: 'middleware:afterHandler',
    chatModel: 'chatModel',
    toolExecution: 'toolExecution',
    errorHandler: 'errorHandler'
};

;// CONCATENATED MODULE: ./src/middleware.ts

class MiddlewareHandler {
    middlewares = [];
    use(middlewares) {
        this.middlewares.push(...middlewares);
    }
    run(context, handler) {
        byorg_utils_namespaceObject.logger.debug('Middleware chain started');
        // Call middlewares in order, and call handler at the end
        const runFn = (i)=>{
            // No more middlewares to run, call the handler
            if (i >= this.middlewares.length) {
                byorg_utils_namespaceObject.logger.debug('Middleware chain finished');
                return handler();
            }
            const nextRun = ()=>runFn(i + 1);
            return this.middlewares[i](context, nextRun);
        };
        return runFn(0);
    }
}

;// CONCATENATED MODULE: ./src/performance.ts
class PerformanceTimeline {
    marks = [];
    measures = [];
    marksTimes = new Map();
    timingOrigin;
    constructor(){
        this.timingOrigin = performance.now();
    }
    mark(name) {
        const startTime = performance.now();
        const entry = {
            entryType: 'mark',
            name,
            startTime
        };
        this.marks.push(entry);
        this.marksTimes.set(name, startTime);
    }
    measure(name, startMark, endMark) {
        const endTime = this._resolveMark(endMark, performance.now());
        const startTime = this._resolveMark(startMark, this.timingOrigin);
        const entry = {
            entryType: 'measure',
            name,
            duration: endTime - startTime,
            startTime
        };
        this.measures.push(entry);
    }
    markStart(name) {
        this.mark(`${name}:start`);
    }
    markEnd(name) {
        this.mark(`${name}:end`);
        this.measure(name, `${name}:start`, `${name}:end`);
    }
    getMarks() {
        return this.marks;
    }
    getMeasures() {
        return this.measures;
    }
    getMeasureTotal(name) {
        const measures = this.measures.filter((measure)=>measure.name === name);
        return measures.reduce((total, measure)=>total + measure.duration, 0);
    }
    toJSON() {
        return {
            marks: [
                ...this.marks
            ],
            measures: [
                ...this.measures
            ]
        };
    }
    _resolveMark(mark, defaultValue) {
        if (mark === undefined) {
            return defaultValue;
        }
        const markTime = this.marksTimes.get(mark);
        if (markTime == null) {
            console.warn(`Performance mark "${mark}" not found, using default value`);
            return defaultValue;
        }
        return markTime;
    }
}

;// CONCATENATED MODULE: ./src/references.ts
const getReferenceStorage = ()=>{
    const referencesKeys = new Set();
    const references = [];
    const getKey = (ref)=>ref.url;
    return {
        addReferences (references) {
            for (const ref of references){
                const refKey = getKey(ref);
                if (!referencesKeys.has(refKey)) {
                    referencesKeys.add(refKey);
                    references.push(ref);
                }
            }
        },
        getReferences () {
            return Object.freeze(references);
        }
    };
};

;// CONCATENATED MODULE: ./src/attachments.ts
const SUPPORTED_ATTACHMENT_TYPES = [
    'image/jpeg',
    'image/png',
    'image/webp',
    'image/gif'
];
function isAttachmentTypeSupported(mimeType) {
    return SUPPORTED_ATTACHMENT_TYPES.includes(mimeType);
}

;// CONCATENATED MODULE: ./src/validation.ts

const requestValidationMiddleware = (context, next)=>{
    validateAttachments(context);
    return next();
};
const validateAttachments = (context)=>{
    if (!context.lastMessage.attachments?.length) {
        return;
    }
    const unsupportedAttachmentsInCommand = context.lastMessage.attachments.filter((attachment)=>!isAttachmentTypeSupported(attachment.mimeType));
    // Unsupported attachment in the last message should stop the processing.
    if (unsupportedAttachmentsInCommand.length > 0) {
        throw new UnsupportedAttachmentsException(unsupportedAttachmentsInCommand);
    }
    // Remove unsupported attachments from previous messages and continue.
    context.messages.slice(0, -1).filter((m)=>!!m.attachments && m.attachments.length > 0).forEach((m)=>{
        m.attachments = m.attachments.filter((attachment)=>isAttachmentTypeSupported(attachment.mimeType));
    });
};
class UnsupportedAttachmentsException extends Error {
    attachments;
    constructor(attachments){
        super(`There are ${attachments.length} attachments of unsupported type.`);
        this.attachments = attachments;
    }
}

;// CONCATENATED MODULE: ./src/application.ts






function createApp(config) {
    const { plugins = [], errorHandler = defaultErrorHandler } = config;
    plugins.forEach((plugin)=>{
        byorg_utils_namespaceObject.logger.debug(`Plugin "${plugin.name}" registered`);
    });
    const middlewares = [
        ...plugins.map((plugin)=>plugin.middleware).filter((middleware)=>middleware != null),
        requestValidationMiddleware
    ];
    const effects = [
        ...plugins.flatMap((plugin)=>plugin.effects ?? [])
    ];
    const tools = [
        ...plugins.flatMap((plugin)=>plugin.tools ?? [])
    ];
    // TODO: Validate
    tools.forEach((tool)=>{
        byorg_utils_namespaceObject.logger.debug(`Tool "${tool.name}" registered`);
    });
    const middlewareExecutor = new MiddlewareHandler();
    middlewareExecutor.use(middlewares);
    return {
        processMessages: async (messages, options)=>{
            messages = normalizeMessages(messages);
            const lastMessage = messages.at(-1);
            byorg_utils_namespaceObject.logger.debug(`Processing message for user: ${lastMessage?.senderId}`);
            const performance = new PerformanceTimeline();
            performance.markStart(PerformanceMarks.processMessages);
            const context = {
                messages,
                get lastMessage () {
                    const lastMessage = this.messages.at(-1);
                    // This will normally be true, unless the middleware changes the messages
                    if (lastMessage?.role !== 'user') {
                        throw new Error('Last message in the "messages" list should be a "UserMessage"');
                    }
                    return lastMessage;
                },
                chatModel: config.chatModel,
                systemPrompt: ()=>typeof config.systemPrompt === 'function' ? config.systemPrompt(context) : config.systemPrompt ?? null,
                onPartialResponse: options?.onPartialResponse,
                tools,
                references: getReferenceStorage(),
                resolvedEntities: {},
                extras: options?.extras ?? {},
                performance
            };
            const handler = async ()=>{
                // Closes the 'middleware:beforeHandler' mark started before middlewareExecutor has run
                performance.markEnd(PerformanceMarks.middlewareBeforeHandler);
                performance.markStart(PerformanceMarks.chatModel);
                const response = await context.chatModel.generateResponse(context);
                performance.markEnd(PerformanceMarks.chatModel);
                // Opens the 'middleware:afterHandler' mark that will be closed after middlewareExecutor has run
                performance.markStart(PerformanceMarks.middlewareAfterHandler);
                return response;
            };
            let response;
            try {
                performance.markStart(PerformanceMarks.middlewareBeforeHandler);
                // markEnd(PerformanceMarks.middlewareBeforeHandler) & markStart(PerformanceMarks.middlewareAfterHandler) happen in handler
                response = await middlewareExecutor.run(context, handler);
                performance.markEnd(PerformanceMarks.middlewareAfterHandler);
            } catch (error) {
                performance.markStart(PerformanceMarks.errorHandler);
                response = await errorHandler(error, context);
                performance.markEnd(PerformanceMarks.errorHandler);
            } finally{
                performance.markEnd(PerformanceMarks.processMessages);
            }
            const pendingEffects = Promise.all(effects.map(async (effect)=>{
                try {
                    return await effect(context, response);
                } catch (error) {
                    byorg_utils_namespaceObject.logger.error(`Effect failed ${effect.name}`, error);
                }
            }));
            return {
                response,
                pendingEffects
            };
        }
    };
}
// Removes trailling assistant messages in order to make UserMessage the last one
function normalizeMessages(messages) {
    const lastUserMessageIndex = messages.findLastIndex((m)=>m.role === 'user');
    if (lastUserMessageIndex === messages.length - 1) {
        return messages;
    }
    const result = messages.slice(0, lastUserMessageIndex + 1);
    byorg_utils_namespaceObject.logger.warn(`Ignored ${messages.length - result.length} trailing assistant message(s).`);
    return result;
}
function defaultErrorHandler(error, _context) {
    byorg_utils_namespaceObject.logger.error('Error while processing message:', error);
    return {
        role: 'system',
        content: 'An error occurred while processing your request. Please try again later.',
        error
    };
}

;// CONCATENATED MODULE: external "ai"
const external_ai_namespaceObject = require("ai");
;// CONCATENATED MODULE: ./src/ai/vercel.ts


// Workaround for memory issue happening when sending image attachment. The attachments get inefficiently serialised causing a memory spike.
const VERCEL_AI_SHARED_OPTIONS = {
    experimental_telemetry: {
        // Make sure telemetry is turned off.
        isEnabled: false,
        // Disable serialization of inputs and outputs to reduce memory usage.
        // This should be automatically disabled, but there is currently a bug in the AI SDK.
        recordInputs: false,
        recordOutputs: false
    }
};
class VercelChatModelAdapter {
    _options;
    constructor(_options){
        this._options = _options;
    }
    async generateResponse(context) {
        let systemPrompt = context.systemPrompt();
        if (systemPrompt) {
            const entitiesPrompt = formatResolvedEntities(context.resolvedEntities);
            if (entitiesPrompt) {
                systemPrompt += '\n\n' + entitiesPrompt;
            }
        }
        const messages = [];
        if (systemPrompt) {
            messages.push({
                role: 'system',
                content: systemPrompt
            });
        }
        messages.push(...context.messages.map(toMessageParam));
        const getRunToolFunction = (tool, context)=>(params)=>{
                context.performance.markStart(PerformanceMarks.toolExecution);
                const result = tool.handler(params, context);
                context.performance.markEnd(PerformanceMarks.toolExecution);
                return result;
            };
        const tools = Object.fromEntries(context.tools.map((tool)=>[
                tool.name,
                (0,external_ai_namespaceObject.tool)({
                    description: tool.description,
                    parameters: tool.parameters,
                    execute: getRunToolFunction(tool, context)
                })
            ]));
        const executionContext = {
            tools,
            messages: messages
        };
        const executionResult = context.onPartialResponse ? await this.executeRequestWithStreaming(executionContext, context.onPartialResponse) : await this.executeRequest(executionContext);
        if (executionResult.finishReason !== 'stop' && executionResult.finishReason !== 'length') {
            throw new AiProviderError(`The response stream ended with an unexpected reason: ${executionResult.finishReason}`);
        }
        const usage = this.getUsageFromExecutionResult(executionResult);
        return {
            role: 'assistant',
            content: executionResult.text,
            usage
        };
    }
    async executeRequestWithStreaming(context, onPartialResponse) {
        const startTime = performance.now();
        const result = await (0,external_ai_namespaceObject.streamText)({
            ...VERCEL_AI_SHARED_OPTIONS,
            model: this._options.languageModel,
            messages: context.messages,
            maxTokens: this._options.maxTokens,
            maxSteps: this._options.maxSteps,
            tools: context.tools
        });
        let currentText = '';
        for await (const textDelta of result.textStream){
            currentText += textDelta;
            onPartialResponse(currentText, textDelta);
        }
        const responseTime = performance.now() - startTime;
        const steps = await result.steps;
        const toolCalls = steps.flatMap((step)=>step.toolCalls);
        // By the time we get here, the stream is already finished.
        return {
            finishReason: await result.finishReason,
            text: await result.text,
            toolCalls,
            usage: await result.usage,
            stepsCount: (await result.steps).length,
            responseTime,
            model: (await result.response).modelId
        };
    }
    async executeRequest(context) {
        const startTime = performance.now();
        const result = await (0,external_ai_namespaceObject.generateText)({
            ...VERCEL_AI_SHARED_OPTIONS,
            model: this._options.languageModel,
            messages: context.messages,
            maxTokens: this._options.maxTokens,
            maxSteps: this._options.maxSteps,
            tools: context.tools
        });
        const responseTime = performance.now() - startTime;
        const toolCalls = result.steps.flatMap((step)=>step.toolCalls);
        return {
            finishReason: result.finishReason,
            text: result.text,
            toolCalls,
            usage: result.usage,
            stepsCount: result.steps.length,
            responseTime,
            model: result.response.modelId
        };
    }
    getUsageFromExecutionResult(result) {
        // TODO: Make inputTokens and outputTokens nullable
        const usage = {
            // Those should include sum of all calls
            inputTokens: toNumberOrZero(result.usage.promptTokens),
            outputTokens: toNumberOrZero(result.usage.completionTokens),
            requests: result.stepsCount,
            responseTime: result.responseTime,
            model: result.model
        };
        result.toolCalls.forEach((toolCall)=>{
            usage.usedTools ??= {};
            usage.usedTools[toolCall.toolName] = (usage.usedTools[toolCall.toolName] ?? 0) + 1;
        });
        return usage;
    }
}
class AiProviderError extends Error {
}
function toMessageParam(message) {
    if (message.role === 'assistant') {
        return {
            role: message.role,
            content: message.content
        };
    }
    return {
        role: message.role,
        content: toUserContent(message)
    };
}
function toUserContent(message) {
    const files = message.attachments ?? [];
    return [
        {
            type: 'text',
            text: message.content
        },
        ...files.map((file)=>({
                type: 'image',
                image: file.buffer,
                mimeType: file.mimeType
            }))
    ];
}
function toNumberOrZero(n) {
    return Number.isNaN(n) ? 0 : n;
}
function formatResolvedEntities(entities) {
    if (Object.keys(entities).length === 0) {
        return null;
    }
    return `### ENTITY DICTIONARY ###\n
${Object.entries(entities).map(([key, value])=>`'${key}' is '${JSON.stringify(value)}'`).join('\n')}
  `;
}

;// CONCATENATED MODULE: external "ts-regex-builder"
const external_ts_regex_builder_namespaceObject = require("ts-regex-builder");
;// CONCATENATED MODULE: ./src/plugins/commands.ts

function createCommandsPlugin({ commands, prefix }) {
    return {
        name: 'commands',
        middleware: async (context, next)=>{
            const extractor = (0,external_ts_regex_builder_namespaceObject.buildRegExp)([
                external_ts_regex_builder_namespaceObject.startOfString,
                (0,external_ts_regex_builder_namespaceObject.zeroOrMore)(external_ts_regex_builder_namespaceObject.whitespace),
                prefix,
                (0,external_ts_regex_builder_namespaceObject.capture)((0,external_ts_regex_builder_namespaceObject.oneOrMore)(/[\w-]/))
            ]);
            const match = context.lastMessage.content.match(extractor);
            const trigger = match?.[1];
            if (!trigger) {
                // Continue middleware chain
                return await next();
            }
            for (let command of commands){
                if (matchCommand(command.trigger, trigger)) {
                    console.log('Matched command:', command.trigger);
                    if (command.isAllowed?.(context) === false) {
                        console.log('No access to command');
                        return {
                            role: 'system',
                            content: buildUnknownCommandsResponse(trigger, commands)
                        };
                    }
                    const response = await command.handler(context);
                    return {
                        role: 'system',
                        content: response
                    };
                }
            }
            return {
                role: 'system',
                content: buildUnknownCommandsResponse(trigger, commands)
            };
        }
    };
}
function buildUnknownCommandsResponse(trigger, commands) {
    const commandsList = commands.map((command)=>`- ${command.trigger}`).join('\n');
    return `Unknown command: ${trigger}.\nHere's a list of possible commands:\n${commandsList}`;
}
function matchCommand(matcher, text) {
    return typeof matcher === 'string' ? text === matcher : matcher.test(text);
}

;// CONCATENATED MODULE: ./src/plugins/logging.ts
const getFormattedNow = ()=>new Date().toISOString();
const loggingPlugin = {
    name: 'logging',
    middleware: async (context, next)=>{
        const serializedCommand = JSON.stringify(context.lastMessage);
        console.debug(`${getFormattedNow()} Handling: "${serializedCommand}"...`);
        try {
            // Continue middleware chain
            const response = await next();
            console.debug(`${getFormattedNow()} Handling: "${serializedCommand}" finished.`);
            return response;
        } catch (error) {
            const errorMessage = error instanceof Error ? error.message : String(error);
            console.error(`${getFormattedNow()} Handling: "${serializedCommand}" failed, error: ${errorMessage}`);
            throw error;
        }
    }
};

;// CONCATENATED MODULE: ./src/mock/mock-model.ts
const LOREM_IPSUM_RESPONSES = [
    'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.',
    'Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat.',
    'Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur.',
    'Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.',
    'Sed ut perspiciatis unde omnis iste natus error sit voluptatem accusantium doloremque laudantium, totam rem aperiam, eaque ipsa quae ab illo inventore veritatis et quasi architecto beatae vitae dicta sunt explicabo.',
    'Nemo enim ipsam voluptatem quia voluptas sit aspernatur aut odit aut fugit, sed quia consequuntur magni dolores eos qui ratione voluptatem sequi nesciunt.',
    'Neque porro quisquam est, qui dolorem ipsum quia dolor sit amet, consectetur, adipisci velit, sed quia non numquam eius modi tempora incidunt ut labore et dolore magnam aliquam quaerat voluptatem.',
    'Ut enim ad minima veniam, quis nostrum exercitationem ullam corporis suscipit laboriosam, nisi ut aliquid ex ea commodi consequatur?',
    'Quis autem vel eum iure reprehenderit qui in ea voluptate velit esse quam nihil molestiae consequatur, vel illum qui dolorem eum fugiat quo voluptas nulla pariatur?'
];
function createMockChatModel(config) {
    const responses = config?.responses ?? LOREM_IPSUM_RESPONSES;
    const delay = config?.delay ?? 100;
    const processRequest = config?.processRequest;
    const calls = [];
    let lastRandom = config?.seed ?? Date.now();
    return {
        calls,
        generateResponse: async (context)=>{
            calls.push([
                context
            ]);
            lastRandom = random(lastRandom);
            const response = processRequest ? processRequest(context) : responses[lastRandom % responses.length];
            const tokens = response.split(/(\S+\s*)/).filter(Boolean);
            if (context.onPartialResponse) {
                let accumulator = '';
                for (const token of tokens){
                    await sleep(delay);
                    accumulator += token;
                    context.onPartialResponse(accumulator, token);
                }
            }
            await sleep(delay);
            const allInputLength = context.messages.reduce((acc, msg)=>acc + msg.content.length, 0);
            return Promise.resolve({
                role: 'assistant',
                content: response,
                usage: {
                    model: 'test',
                    inputTokens: Math.round(allInputLength / 4),
                    outputTokens: tokens.length,
                    requests: 1,
                    responseTime: tokens.length * delay
                }
            });
        }
    };
}
function sleep(ms) {
    return new Promise((resolve)=>setTimeout(resolve, ms));
}
// Simple PRNG for testing
// Definitely not safe for production
function random(state) {
    return state * 16807 % 2147483647; // Linear congruential generator
}

;// CONCATENATED MODULE: ./src/index.ts











var __webpack_export_target__ = exports;
for(var __webpack_i__ in __webpack_exports__) __webpack_export_target__[__webpack_i__] = __webpack_exports__[__webpack_i__];
if(__webpack_exports__.__esModule) Object.defineProperty(__webpack_export_target__, '__esModule', { value: true });
